*=================================================================;
*Compression and P3D;
*=================================================================;
*Objective: To demonstrate the implementation of compression and P3D;
*Author: Zhiwu Zhang;
*Version: 2.10;
*Last update: march 1, 2010;

*Instruction:
*1. This set of SAS code and data serve the demonstration of using P3D and compression. The SAS code can not be used for your own data without modification.
*   In addition to path and file names defined in step 0 and 2, the modifications in Step 3-6 may be required for modeling (nmaes of phenotype,fators etc)

*2. The demonstation is also avaiable from the tutorial dataset of the TASSEL software package:
*   Bradbury, P.J. et al. TASSEL: software for association mapping of complex traits in diverse samples. Bioinformatics 23, 2633-2635 (2007).

*3. This demonstration should be run with two Stage. 

*Stage I: Grouping optimization;
	*The optimization is to evaluate cluster algorithms and compression levels to achieve optimum model file (maximum likelihood);
	*Set parameters as follows:;
	*ClusterMeth=AVE;*Try each of: AVE, CEN, COM, FLE, MCQ, MED, SIN, WAR;
	*grpStartAt=1;
	*grpEndA=51
	*snpStartAt=1;
	*snpEndAt=1;
	*OPTAppraoch=0;

*Stage II: GWAS;
	*Set parameters as follows:;
	*ClusterMeth=AVE;*Use the best of: AVE, CEN, COM, FLE, MCQ, MED, SIN, WAR;
	*grpStartAt=0;
	*grpEndA=0
	*snpStartAt=1;
	*snpEndAt=553;*the total number of markers;
	*OPTAppraoch=2;*use 1-5, 2 is recommended:
	*CF=5;*set it to the compression leve with the best model fit; 

*-------------------------------------------------------------;
*Step 0: SAS macro used;
*-------------------------------------------------------------;
DM 'ODSRESULTS' CLEAR EDITOR;  
DM 'clear log; clear output;';
options nodate nocenter ls=150 ps=32767;
Title "Compression and P3D";
%inc 'C:\Current\ComP3D\Matrixconvert.sas';
%inc 'C:\Current\ComP3D\LORG.sas';
%inc 'C:\Current\ComP3D\setParms.sas';
%inc 'C:\Current\ComP3D\getAvgKin.sas';
%inc 'C:\Current\CAS\Code\hetGenVar.SAS';

*-------------------------------------------------------------;
*Step 1: Configuration;
*-------------------------------------------------------------;
%global myExcelPheno myExcel Va Ve OPTAppraoch;
%global snpStartAt snpEndAt;
%global grpStartAt grpEndAt;

%let nTaxa=277; *number of indiviudals in the kinship file;
%let acceleration=.1; *the rate to determine the interval to optimize number of groups;

%let ClusterMeth=AVE;*choose one of: AVE, CEN, COM, FLE, MCQ, MED, SIN, WAR;
%let grpStartAt=0;*Change to 1 for optimization and 0 for GWAS;
%let grpEndAt=0;*Change to 51 for optimization and 0 for GWAS;
%let OPTAppraoch=2; /*0. reduce mdoel only, 1. Full optimization, 2.  P3D, 3. Residual, 4. BLUP, 5. Regression on BLUP */;
%let CF=3; 

%let snpStartAt=1;
%let snpEndAt=553;*Change to 553 blocks for the whole analysis;

%let myTxt_type3="C:\Current\ComP3D\Results\Demo_type3.csv" ;
%let myTxt_sol=  "C:\Current\ComP3D\Results\Demo_sol.csv" ;
%let myTxt_fit=  "C:\Current\ComP3D\Results\Demo_fit.csv" ;
%let myTxt_cov=  "C:\Current\ComP3D\Results\Demo_cov.csv" ;
%let fileDump=   "C:\Current\ComP3D\Results\Demo_dum.csv" ;

%let nSampled=%SYSEVALF(&nTaxa/&CF);
%put &nSampled;

*-------------------------------------------------------------;
*Step 2: Load data;
*-------------------------------------------------------------;

DATA Kin;
attrib Taxa length=$15;
infile 'C:\Current\ComP3D\data\kin_SNP553_taca277_from_SAS_All.txt' dlm='09'x DSD MISSOVER firstobs =2 lrecl=20000;
input Taxa taxa1-taxa277;
RUN;


Data perf3Traits; 
attrib taxa length=$15;
infile 'C:\Current\ComP3D\data\three_traits_rn_rename.txt' delimiter='09'x firstobs=3 lrecl=20000;
input taxa$ EarHT	dpoll	EarDia;
RUN;

Data STRUCT ;
attrib taxa length=$15;
infile 'C:\Current\ComP3D\data\popStructure_taxa286_rn_rename.txt'  delimiter='09'x firstobs=4;
input taxa$ Q1-q3;
RUN;

DATA SNP;
attrib taxa length=$15;
infile 'C:\Current\ComP3D\data\SNP553_taca277_from_SAS_major_chr.txt' dlm='09'x  firstobs =2 lrecl=30000;
Array SNP{553}$  SNP1-SNP553;
input taxa SNP1-SNP553;
RUN;

data G; 
   input col1; 
   datalines; 
1
; 
run; 
*-------------------------------------------------------------;
*Step 3: Cluster based on kinship;
*-------------------------------------------------------------;

/*convert correlation to distance: substract kinship from the maximum (2)*/
%Macro cor2dis(cor, dis,nTaxa);
data &dis;
set &cor;
%do i=1 %to &nTaxa;
	taxa&i=2-taxa&i;
%end;
run;
%mend;
%cor2dis(kin,myDis,&NTaxa)

Data dist(type=distance); 
attrib ID length=$8;
set myDis;
RUN;

ods output "Cluster History"=ClusterHistory;
proc cluster
   data=dist method=AVE  pseudo outtree=tree ; 
    id taxa; 
run; 
ods output close;



proc tree data=tree horizontal spaces=2  n=&nSampled out=mycluster ; 
   id taxa; 
run; 

*-------------------------------------------------------------;
*Step 4: Calculate kinship among groups;
*-------------------------------------------------------------;

data kinInd;set kin;drop taxa;run;
data IndID;set dist;order=_n_;keep taxa order;run;
proc sort data=IndID;by taxa;run;proc sort data=mycluster;by taxa;run;
data cluster;merge IndID mycluster;by taxa;run;
data clusterID;set cluster;keep CLUSTER;run;

%getAvgKin(kinInd,clusterID,kinCluster)

*-------------------------------------------------------------;
*Step 5: Insert Cluster ID into phenotype and genotype data;
*-------------------------------------------------------------;

data IDNum;set cluster;ID=cluster;keep TAXA ID;run;
proc sort data=IDNum out=IDNumsort;by Taxa;run;
proc sort data=perf3Traits out=perf3TraitsSort;by Taxa;run;
proc sort data=struct out=structsort;by Taxa;run;
proc sort data=SNP out=SNPSort;by Taxa;run;
data perfRaw;merge IDNumsort structsort perf3TraitsSort SNPSort;by Taxa;run;
proc sort data=perfRaw;by ID;run;

*-------------------------------------------------------------;
*Step 6: GWAS with MLM;
*-------------------------------------------------------------;
%Matrixconvert(Matrix2d=kinCluster,Matrix1d=myDataRelation,direction=1)


data perfpre;
	set perfRaw;
	if id>0;
	trait=dpoll;
	if trait>0;
	if Q1;
	IDOrder+1;
run;

*********Build Genetic variace and covariance matrix************;
proc sort data=perfpre;by ID;run;
data RANEFF; set perfpre; keep ID; run; 
%LORG(DataRelation=myDataRelation,DataRandom=RANEFF,dataLorG=G,LORGType='U',DataG=MYLORGDATA);

PROC SORT DATA=cluster out=clusterPop NODUPKEY;
BY cluster;
RUN;

data clusterPop2;set clusterPop;pop=ROUND(1+0*UNIFORM(-1),1.0); keep cluster pop;run;
%inc 'C:\Current\sasmacro\hetGenVar.SAS';
%hetGenVar(randomPop=clusterPop2,dataLorG=MYLORGDATA,DATALORG2=MYLORGDATA2);


data perfpre;set perfpre;drop snp2-snp553;run;
data test0;set perfpre;wt=1;if _n_<=100;pop=0;run;
data test1;set perfpre;wt=0.5;if _n_>0 ;pop=1;run;
data test2;set perfpre;wt=0.5;if _n_>0 ;pop=2;run;
data test;set test0  test1 test2;newID=_n_;run;
data test12;set   test1 test2;newID=_n_;run;

*following two give the same;

Proc mixed data=test1 METHOD=REML noprofile covtest itdetails MAXITER=20 /*CONVH=0.00000001*/ ;
    class ID pop; 
    model trait=Q1-Q2 /solution  htype=1,3 DDFM=RESIDUAL Chisq outp=mypred ;
 	random ID /type=lin(1) lDATA=MYLORGDATA2 ;
	*parms 8.4 14 /hold=1 2;
	*repeated pop;
	*weight wt;
run;

Proc mixed data=test12 METHOD=REML noprofile covtest itdetails MAXITER=20 /*CONVH=0.00000001*/ ;
    class ID pop; 
    model trait=Q1-Q2 /solution  htype=1,3 DDFM=RESIDUAL Chisq outp=mypred ;
 	random ID /type=lin(1) lDATA=MYLORGDATA2 ;
	repeated id/group=pop;
run;


Proc mixed data=test METHOD=REML noprofile covtest itdetails MAXITER=20 /*CONVH=0.00000001*/ ;
    class ID pop; 
    model trait=Q1-Q2 /solution  htype=1,3 DDFM=RESIDUAL Chisq outp=mypred ;
 	random ID /type=lin(1) lDATA=MYLORGDATA2 ;
	*parms 8.4 14 /hold=1 2;
	repeated id/group=pop;
	weight wt;
run;


Proc mixed data=perfpre METHOD=REML noprofile covtest itdetails MAXITER=20 /*CONVH=0.00000001*/ ;
    class ID; 
    model trait=Q1-Q2 /solution  htype=1,3 DDFM=RESIDUAL Chisq outp=mypred ;
 	random ID /type=lin(1) lDATA=MYLORGDATA2 ;
	*parms 8.4 14 /hold=1 2;
run;

