/**********************************************************************
                               Macro Matrix2dto1d
AUTHOR:
     Zhiwu Zhang (zz19@cornell.edu), April 2005.

PURPOSE:
     Convert matrix from 1 dimention to two dimension (direction=2) or reverse (direction=1).

REQUIRES:
     Requires Base SAS and SAS/IML software, Version 8 or later.

INPUT and output: 
	direction -	SAS dataset of matrix in 2 dimentions
INPUT and output: 
	Matrix2d -	SAS dataset of matrix in 2 dimentions
	Matrix1d -	SAS dataset of matrix in one dimention (up triangle)
/*********************************************************************/

%macro Matrixconvert(Matrix2d=,Matrix1d=,direction=);
proc iml;
start main;

*convert SAS dataset to IML array;

*-----------------------------------------------------;
*2d to 1d;
*-----------------------------------------------------;
if(&direction=1) then
do;
use &Matrix2d;
read all into m2d;
nid = nrow(m2d);

*determine one way array size;
onewaynid=nid*(nid+1)/2;
*nid=(sqrt(1+8*onewaynid)-1)/2;

*create one way array;
AAA = j(onewaynid,1,0);

*Transfer two way array to one way array;
do i = 1 to nid;
	do j = 1 to i;
			inddexIJ=i*(i-1)/2+j;
			AAA[inddexIJ] = m2d[i,j];
	end;
end;
*Create SAS data set;
cnames={val};
create &Matrix1d from AAA[colname=cnames] ;
append from AAA;
end;

*-----------------------------------------------------;
*1d to 2d;
*-----------------------------------------------------;
if(&direction=2) then
do;
use &Matrix1d;
read all into AAA;
onewaynid = nrow(AAA);

*determine two way array size;
nid=(sqrt(1+8*onewaynid)-1)/2;

*create one way array;
m2d = j(nid,nid,0);

*Transfer two way array to one way array;
do i = 1 to nid;
	do j = 1 to i;
			inddexIJ=i*(i-1)/2+j;
			m2d[i,j]=AAA[inddexIJ];
			m2d[j,i]=AAA[inddexIJ];
	end;
end;
*Create SAS data set;
*cnames={val};
create &Matrix2d from m2d ;
append from m2d;
end;

finish;
run main;
quit;
%mend;
